import argparse
import copy
import os
import sys
import unittest

from integration.partial.integration_test_utils import IntegrationTestUtils


class FullTestSurvey(unittest.TestCase):

    TEST_OUT_PATH: str = "NOT_SET"
    TEST_START_DATE: str = 'NOT_SET'
    TEST_JOB_DIR: str = "NOT_SET"

    @classmethod
    def setUpClass(cls) -> None:
        FullTestSurvey.write_temp_run_config_file()
        FullTestSurvey.run_survey_pipeline()


    @staticmethod
    def write_temp_run_config_file():
        default_config = IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH
        default_config_dict: dict = IntegrationTestUtils.load_json_file(default_config)
        run_dict: dict = copy.deepcopy(default_config_dict)
        run_dict['WorkspacePathout'] = FullTestSurvey.TEST_OUT_PATH
        run_dict['WorkspacePath'] = FullTestSurvey.TEST_OUT_PATH
        # run_dict['Survey']['SkipServerDownload'] = True
        run_dict['Survey']['SkipServerDownload'] = False
        # run_dict['ServerName'] = ''  # nothing, as local machine

        IntegrationTestUtils.TEMP_CONFIG_FILE_NAME = FullTestSurvey.TEST_OUT_PATH + "temp_config.json"
        IntegrationTestUtils.write_json_file(run_dict, IntegrationTestUtils.TEMP_CONFIG_FILE_NAME)

    # @staticmethod
    # def unpack_dependencies():
    #     IntegrationTestUtils.unpack_zip(IntegrationTestUtils.EXAMPLE_SURVEY_FILE_PATH, FullTestSurvey.TEST_OUT_PATH)

    @staticmethod
    def run_survey_pipeline():
        component = 'Survey'
        IntegrationTestUtils.run_external_pipeline(component,
                                                   FullTestSurvey.TEST_START_DATE,
                                                   IntegrationTestUtils.EMAIL_CRED_PATH)


    def test_standard_run_input_status_success(self):
        status_file_path = os.path.join(FullTestSurvey.TEST_OUT_PATH, FullTestSurvey.TEST_JOB_DIR, "STATUS_SUCCESS")
        success_file_exists: bool = os.path.isfile(status_file_path)
        self.assertTrue(success_file_exists)

    def test_standard_run_input_merged_survey_created(self):
        status_file_path = os.path.join(FullTestSurvey.TEST_OUT_PATH, FullTestSurvey.TEST_JOB_DIR,
                                        "ExportCSV", "Merged_SurveyData.csv")
        success_file_exists: bool = os.path.isfile(status_file_path)
        file_not_empty: bool = IntegrationTestUtils.check_file_not_empty(status_file_path)
        self.assertTrue(success_file_exists)
        self.assertTrue(file_not_empty)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--config', required = True)
    parser.add_argument('--outdir', required = True)
    parser.add_argument('--email_cred', required = True)
    parser.add_argument('--run_date_type', required = False)
    parser.add_argument('--custom_run_date', required = False)
    parser.add_argument('unittest_args', nargs='*')

    _args = parser.parse_args()
    _config_file: str = _args.config
    _outdir: str = _args.outdir
    _email_cred_path: str = _args.email_cred
    _run_date_type: str = _args.run_date_type
    _custom_run_date: str = _args.custom_run_date

    IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH = _config_file

    FullTestSurvey.TEST_OUT_PATH = _outdir
    IntegrationTestUtils.EMAIL_CRED_PATH = _email_cred_path

    FullTestSurvey.TEST_START_DATE = IntegrationTestUtils.generate_run_date(_run_date_type, _custom_run_date)

    # Now set the sys.argv to the unittest_args (leaving sys.argv[0] alone)
    sys.argv[1:] = _args.unittest_args
    unittest.main()
